==> apache-opendal-bindings-cpp-0.45.6-src.tar.gz: apache-opendal-bindings-cpp-0.45.6-src/core/src/raw/oio/list/api.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use std::fmt::Display;
use std::fmt::Formatter;
use std::future::Future;
use std::ops::DerefMut;

use crate::raw::oio::Entry;
use crate::raw::*;
use crate::*;

/// PageOperation is the name for APIs of lister.
#[derive(Debug, Copy, Clone, Hash, Eq, PartialEq)]
#[non_exhaustive]
pub enum ListOperation {
    /// Operation for [`List::next`]
    Next,
    /// Operation for [`BlockingList::next`]
    BlockingNext,
}

impl ListOperation {
    /// Convert self into static str.
    pub fn into_static(self) -> &'static str {
        self.into()
    }
}

impl Display for ListOperation {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.into_static())
    }
}

impl From<ListOperation> for &'static str {
    fn from(v: ListOperation) -> &'static str {
        use ListOperation::*;

        match v {
            Next => "List::next",
            BlockingNext => "List::next",
        }
    }
}

/// The boxed version of [`List`]
pub type Lister = Box<dyn ListDyn>;

/// Page trait is used by [`raw::Accessor`] to implement `list` operation.
pub trait List: Unpin + Send + Sync {
    /// Fetch a new page of [`Entry`]
    ///
    /// `Ok(None)` means all pages have been returned. Any following call
    /// to `next` will always get the same result.
    fn next(&mut self) -> impl Future<Output = Result<Option<Entry>>> + MaybeSend;
}

impl List for () {
    async fn next(&mut self) -> Result<Option<Entry>> {
        Ok(None)
    }
}

impl<P: List> List for Option<P> {
    async fn next(&mut self) -> Result<Option<Entry>> {
        match self {
            Some(p) => p.next().await,
            None => Ok(None),
        }
    }
}

pub trait ListDyn: Unpin + Send + Sync {
    fn next_dyn(&mut self) -> BoxedFuture<Result<Option<Entry>>>;
}

impl<T: List + ?Sized> ListDyn for T {
    fn next_dyn(&mut self) -> BoxedFuture<Result<Option<Entry>>> {
        Box::pin(self.next())
    }
}

impl<T: ListDyn + ?Sized> List for Box<T> {
    async fn next(&mut self) -> Result<Option<Entry>> {
        self.deref_mut().next_dyn().await
    }
}

/// BlockingList is the blocking version of [`List`].
pub trait BlockingList: Send {
    /// Fetch a new page of [`Entry`]
    ///
    /// `Ok(None)` means all pages have been returned. Any following call
    /// to `next` will always get the same result.
    fn next(&mut self) -> Result<Option<Entry>>;
}

/// BlockingLister is a boxed [`BlockingList`]
pub type BlockingLister = Box<dyn BlockingList>;

impl<P: BlockingList + ?Sized> BlockingList for Box<P> {
    fn next(&mut self) -> Result<Option<Entry>> {
        (**self).next()
    }
}

impl BlockingList for () {
    fn next(&mut self) -> Result<Option<Entry>> {
        Ok(None)
    }
}

impl<P: BlockingList> BlockingList for Option<P> {
    fn next(&mut self) -> Result<Option<Entry>> {
        match self {
            Some(p) => p.next(),
            None => Ok(None),
        }
    }
}

==> apache-opendal-bindings-cpp-0.45.24-src.tar.gz: apache-opendal-bindings-cpp-0.45.24-src/core/src/raw/oio/list/api.rs <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

use std::future::Future;
use std::ops::DerefMut;

use crate::raw::oio::Entry;
use crate::raw::*;
use crate::*;

/// The boxed version of [`List`]
pub type Lister = Box<dyn ListDyn>;

/// Page trait is used by [`raw::Accessor`] to implement `list` operation.
pub trait List: Unpin + Send + Sync {
    /// Fetch a new page of [`Entry`]
    ///
    /// `Ok(None)` means all pages have been returned. Any following call
    /// to `next` will always get the same result.
    fn next(&mut self) -> impl Future<Output = Result<Option<Entry>>> + MaybeSend;
}

impl List for () {
    async fn next(&mut self) -> Result<Option<Entry>> {
        Ok(None)
    }
}

impl<P: List> List for Option<P> {
    async fn next(&mut self) -> Result<Option<Entry>> {
        match self {
            Some(p) => p.next().await,
            None => Ok(None),
        }
    }
}

/// ListDyn is the dyn version of [`List`]. Makes it possible to use as
/// `Box<dyn ListDyn>`.
pub trait ListDyn: Unpin + Send + Sync {
    /// The dyn version of [`List::next`].
    fn next_dyn(&mut self) -> BoxedFuture<'_, Result<Option<Entry>>>;
}

impl<T: List + ?Sized> ListDyn for T {
    fn next_dyn(&mut self) -> BoxedFuture<'_, Result<Option<Entry>>> {
        Box::pin(self.next())
    }
}

impl<T: ListDyn + ?Sized> List for Box<T> {
    async fn next(&mut self) -> Result<Option<Entry>> {
        self.deref_mut().next_dyn().await
    }
}
